package app;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import enums.MessageType;
import gui.MainWindow;
import interfaces.MessageConsumer;
import interfaces.MessageProducer;

public class MessageConsumerImpl implements MessageConsumer, Runnable {

	// Variables
	private BlockingQueue<String> sharedQueue;
	private MessageProducer[] producers;
	private MainWindow mainWindow;
	private Thread thread;

	// Constants
	private final static String THREAD_NAME_CONSUMER = "consThr";
	private final static String THREAD_NAME_PRODUCER = "prodThr";

	// Constructor
	public MessageConsumerImpl(MainWindow mainwindow) {
		// Inject reference to the GUI (to send messages to the log box).
		this.mainWindow = mainwindow;

		// Create queue to store the messages in.
		sharedQueue = new LinkedBlockingQueue<>();
	}

	///////////////////
	// THREAD CONTROL

	@Override
	public void run() {
		while (true) {
			try {
				// Write to the GUI in a synchronized fashion.
				mainWindow.writeToLog(sharedQueue.take());
			} catch (InterruptedException e) {
				System.err.println("Thread exception in: Consumer");
			}
		}
	}

	@Override
	public void start(int numberOfProducers) {
		if (thread == null) {
			// Create Thread for the consumer.
			thread = new Thread(this, THREAD_NAME_CONSUMER);
			thread.start();

			// Create Threads for the producers and store them
			// (for pause/resume).
			producers = new MessageProducer[numberOfProducers];

			for (int i = 1; i <= numberOfProducers; i++) {
				MessageProducer messageProducer = new MessageProducerImpl(sharedQueue, THREAD_NAME_PRODUCER + i);
				messageProducer.start();
				producers[i - 1] = messageProducer;
			}
		}
	}

	///////////////////
	// CONSUMER CONTROL

	@Override
	public void enqueue(String message, MessageType type) {
		// Assemble output string.
		String messageToAdd = "<" + type.toString() + "> " + message;

		// Add given message to the tail of the queue.
		try {
			sharedQueue.put(messageToAdd);
		} catch (InterruptedException e) {
			System.err.println("Thread exception in: Manual input");
		}
	}

	@Override
	public void setAllProducersEnabled(boolean state) {
		if (state) {
			// Resume all stored threads.
			for (MessageProducer producer : producers) {
				producer.start();
			}
		} else {
			// Pause all stored threads.
			for (MessageProducer generator : producers) {
				generator.pause();
			}
		}
	}
}
