package app;

import java.util.Random;
import java.util.concurrent.BlockingQueue;

import enums.MessageType;
import interfaces.MessageProducer;

class MessageProducerImpl implements MessageProducer, Runnable {

	// Variables
	private BlockingQueue<String> sharedQueue;

	private Thread thread;
	private String threadname;
	private volatile boolean paused;

	private String currentmessage;
	private MessageType messagetype;
	private int sleeptime;

	private Random random;
	private MessageType[] messagetypes;

	// Constants
	private static final int MIN_SLEEP_TIME = 1000;
	private static final int MAX_SLEEP_TIME = 5000;

	// Constructor
	public MessageProducerImpl(BlockingQueue<String> sharedQueue, String threadname) {
		this.sharedQueue = sharedQueue;
		this.threadname = threadname;

		// Set all new threads to enabled.
		paused = false;

		// Create random value generator and fetch enum list
		random = new Random();
		messagetypes = MessageType.values();

		// Create first message.
		produce();
	}

	///////////////////
	// THREAD CONTROL

	@Override
	public void run() {
		// Endless loop to keep the thread alive.
		while (true) {

			// When the thread is enabled, produce new message and store it to
			// message queue.
			if (!paused) {
				produce();
				try {
					sharedQueue.put("<" + messagetype.toString() + "> " + currentmessage);
					Thread.sleep(sleeptime);
				} catch (InterruptedException e) {
					System.err.println("Thread exception in: " + threadname);
				}
			}
		}
	}

	@Override
	public void start() {
		// Create thread object if it isn't present yet and start it.
		// Resume if it's present and set to paused.
		if (thread == null) {
			System.out.println("Creating: " + threadname);
			thread = new Thread(this, threadname);
			thread.start();
		} else if (paused) {
			System.out.println("Resuming: " + threadname);
			paused = false;
		}

	}

	@Override
	public void pause() {
		// Set thread to paused if it isn't already.
		if (thread != null && !paused) {
			System.out.println("Pausing: " + threadname);
			paused = true;
		}
	}

	///////////////////
	// PRODUCER CONTROL

	/**
	 * Manually generates a new message.
	 */
	private void produce() {
		// Generate a random message. (Just a few numbers.)
		currentmessage = String.valueOf(random.nextInt(Integer.MAX_VALUE));

		// Generate a random MessageType.
		messagetype = messagetypes[random.nextInt(messagetypes.length)];

		// Generate a random time interval until the next produced message.
		sleeptime = MIN_SLEEP_TIME + random.nextInt(MAX_SLEEP_TIME - MIN_SLEEP_TIME);
	}
}
